searchData={"items":[{"type":"behaviour","title":"BroadwayRabbitMQ.ChannelPool","doc":"A behaviour used to implement custom AMQP channel pools.\n\nBy default, BroadwayRabbitMQ handles its own pool of AMQP connections and channels. However,\nthere might be use cases where you want to use your own existing pool or custom pooling logic.\nIn those cases, you can implement a custom channel pool using this behaviour.\n\nTo use a custom pool, pass `{:custom_pool, module, args}` as the value of the `:connection`\noption in `BroadwayRabbitMQ.Producer`. `module` needs to implement this behaviour, and `args` is\npassed down to `c:checkout_channel/1` and `c:checkin_channel/2`.","ref":"BroadwayRabbitMQ.ChannelPool.html"},{"type":"behaviour","title":"Examples - BroadwayRabbitMQ.ChannelPool","doc":"Imagine we pass this option when starting the producer:\n\n    connection: {:custom_pool, MyPool, _amqp_connection = :big_pool}\n\nThen, we could define the custom pool as:\n\n    defmodule MyPool do\n      @behaviour BroadwayRabbitMQ.ChannelPool\n\n      @impl true\n      def checkout_channel(name) do\n        conn = %AMQP.Connection{pid: Process.whereis(name)}\n\n        case AMQP.Channel.open(conn) do\n          {:ok, channel} -> {:ok, channel}\n          {:error, reason} -> {:error, %RuntimeError{message: inspect(reason)}}\n        end\n      end\n\n      @impl true\n      def checkin_channel(_name, channel) do\n        case AMQP.Channel.close(channel) do\n          :ok -> :ok\n          {:error, reason} -> {:error, %RuntimeError{message: inspect(reason)}}\n        end\n      end\n    end","ref":"BroadwayRabbitMQ.ChannelPool.html#module-examples"},{"type":"callback","title":"BroadwayRabbitMQ.ChannelPool.checkin_channel/2","doc":"Invoked to check a channel back into the pool.\n\n`channel` is a channel that was returned by `c:checkout_channel/1`.\n\nIf there is an error, you can return any exception as `{:error, exception}`.\n\nIn case your pool fails to handle this properly, BroadwayRabbitMQ will try to close the channel\nby itself using `AMQP.Channel.close/1`.","ref":"BroadwayRabbitMQ.ChannelPool.html#c:checkin_channel/2"},{"type":"callback","title":"BroadwayRabbitMQ.ChannelPool.checkout_channel/1","doc":"Invoked to check out a AMQP channel from the pool.\n\nIf there is an error, you can return any exception as `{:error, exception}`.\n\nThis callback is invoked from a `BroadwayRabbitMQ.Producer` process. If you need\nthe PID of that process, call `self()` in your implementation. This can be useful\nfor things like linking or monitoring.","ref":"BroadwayRabbitMQ.ChannelPool.html#c:checkout_channel/1"},{"type":"module","title":"BroadwayRabbitMQ.Producer","doc":"A RabbitMQ producer for Broadway.","ref":"BroadwayRabbitMQ.Producer.html"},{"type":"module","title":"Features - BroadwayRabbitMQ.Producer","doc":"* Automatically acknowledges/rejects messages.\n  * Handles connection outages using backoff for retries.\n\nFor a quick getting started on using Broadway with RabbitMQ, please see\nthe [RabbitMQ Guide](https://hexdocs.pm/broadway/rabbitmq.html).","ref":"BroadwayRabbitMQ.Producer.html#module-features"},{"type":"module","title":"Options - BroadwayRabbitMQ.Producer","doc":"* `:buffer_size` - Optional, but required if `:prefetch_count` under `:qos` is\nset to `0`. Defines the size of the buffer to store events without demand.\nCan be `:infinity` to signal no limit on the buffer size. This is used to\nconfigure the GenStage producer, see the `GenStage` docs for more details.\nDefaults to `:prefetch_count * 5`.\n\n  * `:buffer_keep` - Optional. Used in the GenStage producer configuration.\nDefines whether the `:first` or `:last` entries should be kept on the\nbuffer in case the buffer size is exceeded. Defaults to `:last`.\n\n  * `:on_success` - Configures the acking behaviour for successful messages.\nSee the \"Acking\" section below for all the possible values.\nThis option can also be changed for each message through\n`Broadway.Message.configure_ack/2`. The default value is `:ack`.\n\n  * `:on_failure` - Configures the acking behaviour for failed messages.\nSee the \"Acking\" section below for all the possible values.\nThis option can also be changed for each message through\n`Broadway.Message.configure_ack/2`. The default value is `:reject_and_requeue`.\n\n  * `:backoff_min` - The minimum backoff interval (default: `1_000`).\n\n  * `:backoff_max` - The maximum backoff interval (default: `30_000`).\n\n  * `:backoff_type` - The backoff strategy: `:stop` for no backoff and\nto stop, `:exp` for exponential, `:rand` for random, and `:rand_exp` for\nrandom exponential (default: `:rand_exp`). The default value is `:rand_exp`.\n\n\n\nThe following options apply to the underlying AMQP connection:\n\n  * `:queue` - Required. The name of the queue. If `\"\"`, then the queue name will\nbe autogenerated by the server but for this to work you have to declare\nthe queue through the `:declare` option.\n\n  * `:connection` - Defines an AMQP URI (a string), a custom pool, or a set of options used by\nthe RabbitMQ client to open the connection with the RabbitMQ broker.\nTo use a custom pool, pass a `{:custom_pool, module, args}` tuple, see\n`BroadwayRabbitMQ.ChannelPool` for more information. If passing an AMQP URI\nor a list of options, this producer manages the AMQP connection instead.\nSee `AMQP.Connection.open/1` for the full list of connection options. The default value is `[]`.\n\n  * `:qos` - Defines a set of prefetch options used by the RabbitMQ client.\nSee `AMQP.Basic.qos/2` for the full list of options. Note that the\n`:global` option is not supported by Broadway since each producer holds only one\nchannel per connection. The default value is `[]`.\n\n    * `:prefetch_size`\n\n    * `:prefetch_count` - The default value is `50`.\n\n  * `:name` - The name of the AMQP connection to use. The default value is `:undefined`.\n\n  * `:metadata` - The list of AMQP metadata fields to copy (default: `[]`). Note\nthat every `Broadway.Message` contains an `:amqp_channel` in its `metadata` field.\nSee the \"Metadata\" section. The default value is `[]`.\n\n  * `:declare` - A list of options used to declare the `:queue`. The\nqueue is only declared (and possibly created if not already there) if this\noption is present and not `nil`. Note that if you use `\"\"` as the queue\nname (which means that the queue name will be autogenerated on the server),\nthen every producer stage will declare a different queue. If you want all\nproducer stages to consume from the same queue, use a specific queue name.\nYou can still declare the same queue as many times as you want because\nqueue creation is idempotent (as long as you don't use the `passive: true`\noption). For the available options, see `AMQP.Queue.declare/3`, `:nowait` is not supported.\n\n  * `:bindings` - A list of bindings for the `:queue`. This option\nallows you to bind the queue to one or more exchanges. Each binding is a tuple\n`{exchange_name, binding_options}` where so that the queue will be bound\nto `exchange_name` through `AMQP.Queue.bind/4` using `binding_options` as\nthe options. Bindings are idempotent so you can bind the same queue to the\nsame exchange multiple times. The default value is `[]`.\n\n  * `:merge_options` - A function that takes the index of the producer in the\nBroadway topology and returns a keyword list of options. The returned options\nare merged with the other options given to the producer. This option is useful\nto dynamically change options based on the index of the producer. For example,\nyou can use this option to \"shard\" load between a few queues where a subset of\nthe producer stages is connected to each queue, or to connect producers to\ndifferent RabbitMQ nodes (for example through partitioning). Note that the options\nare evaluated every time a connection is established (for example, in case\nof disconnections). This means that you can also use this option to choose\ndifferent options on every reconnections. This can be particularly useful\nif you have multiple RabbitMQ URLs: in that case, you can reconnect to a different\nURL every time you reconnect to RabbitMQ, which avoids the case where the\nproducer tries to always reconnect to a URL that is down.\n\n  * `:after_connect` - A function that takes the AMQP channel that the producer\nis connected to and can run arbitrary setup. This is useful for declaring\ncomplex RabbitMQ topologies with possibly multiple queues, bindings, or\nexchanges. RabbitMQ declarations are generally idempotent so running this\nfunction from all producer stages after every time they connect is likely\nfine. This function can return `:ok` if everything went well or `{:error, reason}`.\nIn the error case then the producer will consider the connection failed and\nwill try to reconnect later (same behavior as when the connection drops, for example).\nThis function is run **before** the declaring and binding queues according to\nthe `:declare` and `:bindings` options (described above).\n\n  * `:consume_options` - Options passed down to `AMQP.Basic.consume/4`. Not all options supported by\n`AMQP.Basic.consume/4` are available here as some options would conflict with\nthe internal implementation of this producer. The default value is `[]`.\n\n    * `:consumer_tag`\n\n    * `:no_local`\n\n    * `:no_ack`\n\n    * `:exclusive`\n\n    * `:arguments`\n\n\n\nNote AMQP provides the possibility to define the AMQP connection globally.\nThis is not supported by Broadway. You must configure the connection\ndirectly in the Broadway pipeline, as shown in the next section.","ref":"BroadwayRabbitMQ.Producer.html#module-options"},{"type":"module","title":"Example - BroadwayRabbitMQ.Producer","doc":"@processor_concurrency 50\n    @max_demand 2\n\n    Broadway.start_link(MyBroadway,\n      name: MyBroadway,\n      producer: [\n        module:\n          {BroadwayRabbitMQ.Producer,\n           queue: \"my_queue\",\n           connection: [\n             username: \"user\",\n             password: \"password\",\n             host: \"192.168.0.10\"\n           ],\n           qos: [\n             # See \"Back-pressure and `:prefetch_count`\" section\n             prefetch_count: @processor_concurrency * @max_demand\n           ],\n           on_failure: :reject_and_requeue},\n        # See \"Producer concurrency\" section\n        concurrency: 1\n      ],\n      processors: [\n        default: [\n          concurrency: @processor_concurrency,\n          # See \"Max demand\" section\n          max_demand: @max_demand\n        ]\n      ]\n    )","ref":"BroadwayRabbitMQ.Producer.html#module-example"},{"type":"module","title":"Producer concurrency - BroadwayRabbitMQ.Producer","doc":"For efficiency, you should generally limit the amount of internal queueing.\nWhenever additional messages are sitting in a busy processor's mailbox, they\ncan't be delivered to another processor which may be available or become\navailable first.\n\nOne possible cause of internal queueing is multiple producers. This is because\neach processor's demand will be sent to all producers. For example, if a\nprocessor demands `2` messages and there are `2` producers, each producer\nwill try to  pull `2` messages and give them to the processor. So the\nprocessor may receive `max_demand *  ` messages.\n\nSetting producer `concurrency: 1` will reduce internal queueing, so this is\nthe recommended setting to start with. **Only increase producer concurrency\nif you can measure performance improvements in your system**. Adding another\nsingle-producer pipeline, or another node running the pipeline, are other\nways you may consider to increase throughput.","ref":"BroadwayRabbitMQ.Producer.html#module-producer-concurrency"},{"type":"module","title":"Back-pressure and `:prefetch_count` - BroadwayRabbitMQ.Producer","doc":"Unlike the BroadwaySQS producer, which polls for new messages,\nBroadwayRabbitMQ receives messages as they are are pushed by RabbitMQ. The\n`:prefetch_count` setting instructs RabbitMQ to [limit the number of\nunacknowledged messages a consumer will have at a given\nmoment](https://www.rabbitmq.com/consumer-prefetch.html) (except with a value\nof `0`, which RabbitMQ treats as infinity).\n\nSetting a prefetch limit creates back-pressure from Broadway to RabbitMQ so\nthat the pipeline is not overwhelmed with messages. But setting the limit too\nlow will limit throughput. For example, if the `:prefetch_count` were `1`,\nonly one message could be processed at a time, regardless of other settings.\n\nAlthough the RabbitMQ client has a default `:prefetch_count` of `0`,\nBroadwayRabbitMQ overwrites the default value to `50`, enabling the\nback-pressure mechanism. **To ensure that all processors in a given pipeline\ncan receive messages, the value should be set to at least `max_demand *\n `**, as in the example above.\n\nIncreasing it beyond that could be helpful if latency from RabbitMQ were\nhigh, and in the long term would not cause the pipeline to receive an unfair\nshare of messages, since RabbitMQ uses round-robin delivery to all\nsubscribers. It could mean that a newly-added subscriber would initially\nreceives no messages, as they would have all been prefetched by the existing\nproducer.\n\nIf you're using batchers, you'll need a larger `:prefetch_count` to allow all\nbatchers and processors to be busy simultaneously. Measure your system to\ndecide what number works best.\n\nYou can define `:prefetch_count` as `0` if you wish to disable back-pressure.\nHowever, if you do this, make sure the machine has enough resources to handle\nthe number of messages coming from the broker, and set `:buffer_size` to an\nappropriate value.","ref":"BroadwayRabbitMQ.Producer.html#module-back-pressure-and-prefetch_count"},{"type":"module","title":"Max demand - BroadwayRabbitMQ.Producer","doc":"The best value for `max_demand` depends on how long your messages take to\nprocess. If processing time is long, consider setting it to `1`. Otherwise,\nthe default value of `10` is a good starting point.\n\nMeasure throughput in your own system to see how this setting affects it.","ref":"BroadwayRabbitMQ.Producer.html#module-max-demand"},{"type":"module","title":"Connection loss and backoff - BroadwayRabbitMQ.Producer","doc":"In case the connection cannot be opened or if an established connection is lost,\nthe producer will try to reconnect using an exponential random backoff strategy.\nThe strategy can be configured using the `:backoff_type` option.","ref":"BroadwayRabbitMQ.Producer.html#module-connection-loss-and-backoff"},{"type":"module","title":"Declaring queues and binding them to exchanges - BroadwayRabbitMQ.Producer","doc":"In RabbitMQ, it's common for consumers to declare the queue they're going\nto consume from and bind it to the appropriate exchange when they start up.\nYou can do these steps (either or both) when setting up your Broadway pipeline\nthrough the `:declare` and `:bindings` options.\n\n    Broadway.start_link(MyBroadway,\n      name: MyBroadway,\n      producer: [\n        module:\n          {BroadwayRabbitMQ.Producer,\n          queue: \"my_queue\",\n          declare: [],\n          bindings: [{\"my-exchange\", []}]},\n        concurrency: 1\n      ],\n      processors: [\n        default: []\n      ]\n    )","ref":"BroadwayRabbitMQ.Producer.html#module-declaring-queues-and-binding-them-to-exchanges"},{"type":"module","title":"Acking - BroadwayRabbitMQ.Producer","doc":"You can use the `:on_success` and `:on_failure` options to control how messages\nare acked on RabbitMQ. By default, successful messages are acked and failed\nmessages are rejected. You can set `:on_success` and `:on_failure` when starting\nthe RabbitMQ producer, or change them for each message through\n`Broadway.Message.configure_ack/2`. You can also ack a message *before* the end of the Broadway\npipeline by using `Broadway.Message.ack_immediately/1`, which determines whether to ack or\nreject based on `:on_success`/`:on_failure` too.\n\nHere is the list of all possible values supported by `:on_success` and `:on_failure`:\n\n  * `:ack` - acknowledge the message. RabbitMQ will mark the message as acked and\n    will not redeliver it to any other consumer. This is done via `AMQP.Basic.ack/3`.\n\n  * `:reject` - rejects the message without requeuing (basically, discards\n     the message).  RabbitMQ will not redeliver the message to any other\n     consumer, but a queue can be configured to send rejected messages to a\n     [dead letter exchange](https://www.rabbitmq.com/dlx.html), where another\n     consumer can see why it was dead lettered, how many times, and so on, and\n     potentially republish it. Rejecting is done through `AMQP.Basic.reject/3`\n     with the `:requeue` option set to `false`.\n\n  * `:reject_and_requeue` - rejects the message and tells RabbitMQ to requeue it so\n    that it can be delivered to a consumer again. `:reject_and_requeue`\n    always requeues the message. If the message is unprocessable, this will\n    cause an infinite loop of retries. Rejecting is done through `AMQP.Basic.reject/3`\n     with the `:requeue` option set to `true`.\n\n  * `:reject_and_requeue_once` - rejects the message and tells RabbitMQ to requeue it\n    the first time. If a message was already requeued and redelivered, it will be\n    rejected and not requeued again. This feature uses Broadway-specific message metadata,\n    not RabbitMQ's dead lettering feature. Rejecting is done through `AMQP.Basic.reject/3`.\n\nIf you pass the `no_ack: true` option under `:consume_options`, then RabbitMQ will consider\nevery message delivered to a consumer as **acked**, so the settings above have no effect.\nIn those cases, calling `Broadway.Message.ack_immediately/1` also has no effect.\n\n#","ref":"BroadwayRabbitMQ.Producer.html#module-acking"},{"type":"module","title":"Choosing the right requeue strategy - BroadwayRabbitMQ.Producer","doc":"Choose the requeue strategy carefully.\n\nIf you set the value to `:reject` or `:reject_and_requeue_once`, make sure you handle failed\nmessages properly, either by logging them somewhere or redirecting them to a dead-letter queue\nfor future inspection. These strategies are useful when you want to implement **at most once**\nprocessing: you want your messages to be processed at most once, but if they fail, you prefer\nthat they're not re-processed. It's common to pair this requeue strategy with the use of\n`Broadway.Message.ack_immediately/1` in order to ack the message before doing any work,\nso that if the consumer loses connection to RabbitMQ while processing, the message will have\nbeen acked and RabbitMQ will not deliver it to another consumer. For example:\n\n    def handle_message(_, message, _context) do\n      Broadway.Message.ack_immediately(message)\n      process_message(message)\n      message\n    end\n\n`:reject_and_requeue` is commonly used when you are implementing **at least once** processing\nsemantics. You want messages to be processed at least once, so if something goes wrong and they\nget rejected, they'll be requeued and redelivered to a consumer.\nWhen using `:reject_and_requeue`, pay attention that requeued messages by default will\nbe instantly redelivered, which may result in very high unnecessary workload.\nOne way to handle this is by using [Dead Letter Exchanges](https://www.rabbitmq.com/dlx.html)\nand [TTL and Expiration](https://www.rabbitmq.com/ttl.html).","ref":"BroadwayRabbitMQ.Producer.html#module-choosing-the-right-requeue-strategy"},{"type":"module","title":"Metadata - BroadwayRabbitMQ.Producer","doc":"You can retrieve additional information about your message by setting the `:metadata` option\nwhen starting the producer. This is useful in a handful of situations like when you are\ninterested in the message headers or in knowing if the message is new or redelivered.\nMetadata is added to the `metadata` field in the `Broadway.Message` struct.\n\nThese are the keys in the metadata map that are *always present*:\n\n  * `:amqp_channel` - It contains the `AMQP.Channel` struct. You can use it to do things\n    like publish messages back to RabbitMQ (for use cases such as RPCs). You *should not*\n    do things with the channel other than publish messages with `AMQP.Basic.publish/5`. Other\n    operations may result in undesired effects.\n\nHere is the list of all possible values supported by `:metadata`:\n\n  * `:delivery_tag` - an integer that uniquely identifies the delivery on a channel.\n    It's used internally in AMQP client library methods, like acknowledging or rejecting a message.\n\n  * `:redelivered` - a boolean representing if the message was already rejected and requeued before.\n\n  * `:exchange` - the name of the exchange the queue was bound to.\n\n  * `:routing_key` - the name of the queue from which the message was consumed.\n\n  * `:content_type` - the MIME type of the message.\n\n  * `:content_encoding` - the MIME content encoding of the message.\n\n  * `:headers` - the headers of the message, which are returned in tuples of type\n    `{String.t(), argument_type(), term()}`. The last value of the tuple is the value of\n    the header. You can find a list of argument types\n    [here](https://hexdocs.pm/amqp/readme.html#types-of-arguments-and-headers).\n\n  * `:persistent` - a boolean stating whether or not the message was published with disk persistence.\n\n  * `:priority` - an integer representing the message priority on the queue.\n\n  * `:correlation_id` - it's a useful property of AMQP protocol to correlate RPC requests.\n    You can read more about RPC in RabbitMQ\n    [here](https://www.rabbitmq.com/tutorials/tutorial-six-python.html).\n\n  * `:message_id` - application specific message identifier.\n\n  * `:timestamp` - a timestamp associated with the message.\n\n  * `:type` - message type as a string.\n\n  * `:user_id` - a user identifier that could have been assigned during message publication.\n    RabbitMQ validated this value against the active connection when the message was published.\n\n  * `:app_id` - publishing application identifier.\n\n  * `:cluster_id` - RabbitMQ cluster identifier.\n\n  * `:reply_to` - name of the reply queue.","ref":"BroadwayRabbitMQ.Producer.html#module-metadata"},{"type":"module","title":"Telemetry - BroadwayRabbitMQ.Producer","doc":"This producer emits a few [Telemetry](https://github.com/beam-telemetry/telemetry)\nevents which are listed below.\n\n  * `[:broadway_rabbitmq, :amqp, :open_connection, :start | :stop | :exception]` spans -\n    these events are emitted in \"span style\" when opening an AMQP connection.\n    See `:telemetry.span/3`.\n\n    All these events have the measurements described in `:telemetry.span/3`. The events\n    contain the following metadata:\n\n    * `:connection_name` - the name of the AMQP connection (or `nil` if it doesn't have a name)\n    * `:connection` - the connection info passed when starting the producer (either a URI\n      or a keyword list of options)\n\n  * `[:broadway_rabbitmq, :amqp, :ack, :start | :stop | :exception]` span - these events\n    are emitted in \"span style\" when acking messages on RabbitMQ. See `:telemetry.span/3`.\n\n    All these events have the measurements described in `:telemetry.span/3`. The events\n    contain no metadata.\n\n  * `[:broadway_rabbitmq, :amqp, :reject, :start | :stop | :exception]` span - these events\n    are emitted in \"span style\" when rejecting messages on RabbitMQ. See `:telemetry.span/3`.\n\n    All these events have the measurements described in `:telemetry.span/3`. The `[..., :start]`\n    event contains the following metadata:\n\n    * `:requeue` - a boolean telling if this \"reject\" is asking RabbitMQ to requeue the message\n      or not.\n\n  * `[:broadway_rabbitmq, :amqp, :connection_failure]` execution - this event is executed when\n    the connection to RabbitMQ fails. See `:telemetry.execute/3`.\n\n    The event contains the following metadata:\n\n    * `:reason` - the reason for the failure.","ref":"BroadwayRabbitMQ.Producer.html#module-telemetry"},{"type":"module","title":"Dead-letter Exchanges - BroadwayRabbitMQ.Producer","doc":"Here's an example of how to use a dead-letter exchange setup with broadway_rabbitmq:\n\n    defmodule MyPipeline do\n      use Broadway\n\n      @queue \"my_queue\"\n      @exchange \"my_exchange\"\n      @queue_dlx \"my_queue.dlx\"\n      @exchange_dlx \"my_exchange.dlx\"\n\n      def start_link(_opts) do\n        Broadway.start_link(__MODULE__,\n          name: __MODULE__,\n          producer: [\n            module: {\n              BroadwayRabbitMQ.Producer,\n              on_failure: :reject,\n              after_connect: &declare_rabbitmq_topology/1,\n              queue: @queue,\n              declare: [\n                durable: true,\n                arguments: [\n                  {\"x-dead-letter-exchange\", :longstr, @exchange_dlx},\n                  {\"x-dead-letter-routing-key\", :longstr, @queue_dlx}\n                ]\n              ],\n              bindings: [{@exchange, []}],\n            },\n            concurrency: 2\n          ],\n          processors: [default: [concurrency: 4]]\n        )\n      end\n\n      defp declare_rabbitmq_topology(amqp_channel) do\n        with :ok <- AMQP.Exchange.declare(amqp_channel, @exchange, :fanout, durable: true),\n             :ok <- AMQP.Exchange.declare(amqp_channel, @exchange_dlx, :fanout, durable: true),\n             {:ok, _} <- AMQP.Queue.declare(amqp_channel, @queue_dlx, durable: true),\n             :ok <- AMQP.Queue.bind(amqp_channel, @queue_dlx, @exchange_dlx) do\n          :ok\n        end\n      end\n\n      @impl true\n      def handle_message(_processor, message, _context) do\n        # Raising errors or returning a \"failed\" message here sends the message to the\n        # dead-letter queue.\n      end\n    end","ref":"BroadwayRabbitMQ.Producer.html#module-dead-letter-exchanges"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.8.1 (2023-11-02) - Changelog","doc":"* Fix compilation warning around the use of `Logger.warn/2`.\n  * Fix a bug with validating a behaviour when implementing multiple behaviours.\n  * Fix closing connection from channel pool (see [#129](https://github.com/dashbitco/broadway_rabbitmq/pull/129)).","ref":"changelog.html#v0-8-1-2023-11-02"},{"type":"extras","title":"v0.8.0 (2023-04-25) - Changelog","doc":"* Add the `BroadwayRabbitMQ.ChannelPool` behaviour.\n  * Add the `[:broadway_rabbitmq, :amqp, :connection_failure]` Telemetry event.\n  * Bump Elixir requirement to 1.8+.\n  * Allow nimble_options 1.0+.","ref":"changelog.html#v0-8-0-2023-04-25"},{"type":"extras","title":"v0.7.2 (2022-01-12) - Changelog","doc":"* Support nimble_options 0.4.0 alongside 0.3.x","ref":"changelog.html#v0-7-2-2022-01-12"},{"type":"extras","title":"v0.7.1 (2021-11-25) - Changelog","doc":"* Add support to AMQP 3.0","ref":"changelog.html#v0-7-1-2021-11-25"},{"type":"extras","title":"v0.7.0 (2021-08-30) - Changelog","doc":"* Add support to AMQP 2.0\n  * Require Broadway 1.0","ref":"changelog.html#v0-7-0-2021-08-30"},{"type":"extras","title":"v0.6.5 (2020-12-11) - Changelog","doc":"* Add support for a few Telemetry events. See the \"Telemetry\" section\n    in the docs for `BroadwayRabbitMQ.Producer`.\n\n  * Add support for `:consume_options` when starting a\n    `BroadwayRabbitMQ.Producer` to pass options down to `AMQP.Basic.consume/4`.","ref":"changelog.html#v0-6-5-2020-12-11"},{"type":"extras","title":"v0.6.4 (2020-11-23) - Changelog","doc":"* Bump nimble_options dependency to 0.3.5 which fixes some deprecation\n    warnings.\n\n  * Fix a few potential RabbitMQ issues like possible connection leaking (see\n    [#83](https://github.com/dashbitco/broadway_rabbitmq/pull/83)).","ref":"changelog.html#v0-6-4-2020-11-23"},{"type":"extras","title":"v0.6.3 (2020-11-19) - Changelog","doc":"* Start using nimble_options for validation. This has no practical\n    consequences on the API but introduces a new dependency in broadway_rabbitmq\n    (which was already used by Broadway).\n  * Raise if acking messages fails. See the discussion in\n    [dashbitco/broadway#208](https://github.com/dashbitco/broadway/issues/208).","ref":"changelog.html#v0-6-3-2020-11-19"},{"type":"extras","title":"v0.6.2 (2020-10-24) - Changelog","doc":"* Deprecate use of a default `:on_failure` option.\n  * Expose always-present `:amqp_channel` metadata containing the `AMQP.Channel`\n    struct.","ref":"changelog.html#v0-6-2-2020-10-24"},{"type":"extras","title":"v0.6.1 (2020-06-05) - Changelog","doc":"* Add support for the `:after_connect` option.\n  * Add `auth_mechanisms` to the supported connection options for RabbitMQ.\n  * Support passing in an AMQP connection name.\n  * Update Broadway requirement to `~> 0.6.0` (it was exactly `0.6.0`) before.","ref":"changelog.html#v0-6-1-2020-06-05"},{"type":"extras","title":"v0.6.0 (2020-02-19) - Changelog","doc":"* Update Broadway requirement to 0.6.0.\n  * Re-initialize client options on every reconnect. This means that the `:merge_options`\n    function is called on every reconnect, allowing to do things such as round-robin\n    on a list of RabbitMQ URLs.\n  * Remove support for the deprecated `:requeue` option. Use `:on_success`/`:on_failure`\n    instead.\n  * Improve logging on RabbitMQ disconnections and reconnections.","ref":"changelog.html#v0-6-0-2020-02-19"},{"type":"extras","title":"v0.5.0 (2019-11-04) - Changelog","doc":"* Add support for configuring acking behaviour using `:on_success` and `:on_failure` options\n  * Add support for declare options `:no_wait` and `:arguments`\n  * Handle `:auth_failure`, `:unknown_host` and `:socket_closed_unexpectedly` errors\n  * Add support for a function as the `:connection`\n  * Add support for `:merge_options` option\n  * Update to Broadway v0.5.0","ref":"changelog.html#v0-5-0-2019-11-04"},{"type":"extras","title":"v0.4.0 (2019-08-06) - Changelog","doc":"* Add `:declare` and `:bindings` options to producers\n  * Handle consumer cancellation by reconnecting","ref":"changelog.html#v0-4-0-2019-08-06"},{"type":"extras","title":"v0.3.0 (2019-06-06) - Changelog","doc":"* Allow overriding `:buffer_size` and `:buffer_keep`\n  * Make `:buffer_size` required if `:prefetch_count` is set to `0`\n  * Allow passing RabbitMQ connection options via an AMQP URI","ref":"changelog.html#v0-3-0-2019-06-06"},{"type":"extras","title":"v0.2.0 (2019-05-09) - Changelog","doc":"* New option `:metadata` that allows users to select which metadata should be retrieved\n    and appended to the message struct\n  * New option `:requeue` that allows users to define a strategy for requeuing failed messages","ref":"changelog.html#v0-2-0-2019-05-09"},{"type":"extras","title":"v0.1.0 (2019-04-09) - Changelog","doc":"* Initial release","ref":"changelog.html#v0-1-0-2019-04-09"}],"content_type":"text/markdown"}